;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "eshell-prompt-extras" "20260402.1141"
  "Display extra information for your eshell prompt."
  '((emacs "25"))
  :url "https://github.com/zwild/eshell-prompt-extras"
  :commit "36504072605a2044cf291d1c2ea987cb898c6394"
  :revdesc "36504072605a"
  :keywords '("eshell" "prompt")
  :authors '(("zwild" . "judezhao@outlook.com"))
  :maintainers '(("Xu Chunyang" . "xuchunyang56@gmail.com")))
